@@ -17,16 +17,12 @@ allowed_cidrs = lst(cvalue(app, "allowed_ingress_cidrs", optional=True))
 reaper_schedule_minutes = int(
 	cvalue(app, "reaper_schedule_minutes", optional=True) or "1"
 )
-reaper_consecutive_periods = int(
-	cvalue(app, "reaper_consecutive_periods", optional=True) or "2"
-)
-reaper_period_seconds = int(cvalue(app, "reaper_period_seconds", optional=True) or "60")
-reaper_min_age_seconds = int(
-	cvalue(app, "reaper_min_age_seconds", optional=True) or "60"
-)
 reaper_max_age_hours = float(
 	cvalue(app, "reaper_max_age_hours", optional=True) or "1.0"
 )
+reaper_deployment_timeout = float(
+	cvalue(app, "reaper_deployment_timeout", optional=True) or "1.0"
+)
 
 BaselineStack(
 	app,
@@ -39,10 +35,8 @@ BaselineStack(
 	certificate_arn=certificate_arn,
 	allowed_ingress_cidrs=allowed_cidrs,
 	reaper_schedule_minutes=reaper_schedule_minutes,
-	reaper_consecutive_periods=reaper_consecutive_periods,
-	reaper_period_seconds=reaper_period_seconds,
-	reaper_min_age_seconds=reaper_min_age_seconds,
 	reaper_max_age_hours=reaper_max_age_hours,
+	reaper_deployment_timeout=reaper_deployment_timeout,
 )
 
 app.synth()
