@@ -4,11 +4,13 @@ from __future__ import annotations
 
 import asyncio
 import base64
+import hashlib
 from datetime import UTC, datetime
 from pathlib import Path
-from typing import cast
+from typing import Any, cast
 
 import boto3
+import httpx
 from botocore.exceptions import ClientError
 
 from pulse_aws.baseline import BaselineStackOutputs
@@ -104,18 +106,6 @@ async def _prompt_retry(
 	return True
 
 
-def _prepare_context(
-	context: DeploymentContext | None,
-	reporter: Reporter | None,
-) -> DeploymentContext:
-	"""Normalize context/reporter configuration."""
-	if context is None:
-		return create_context(reporter=reporter)
-	if reporter is not None:
-		context.reporter = reporter
-	return context
-
-
 def _emit_dns_instructions(config: DnsConfiguration, reporter: Reporter) -> None:
 	"""Print DNS instructions via the reporter."""
 	instructions = config.format_for_display()
@@ -123,6 +113,46 @@ def _emit_dns_instructions(config: DnsConfiguration, reporter: Reporter) -> None
 		reporter.info(line)
 
 
+async def _verify_alb_via_https(
+	domain: str,
+	baseline: BaselineStackOutputs,
+) -> bool:
+	"""Verify that the domain reaches the ALB by checking the verification endpoint.
+
+	This works even when Cloudflare proxy is enabled, as it uses HTTPS to verify
+	that requests to the domain actually reach our ALB. Uses a hash token instead
+	of exposing the ALB DNS name for security.
+
+	Args:
+	    domain: The domain to check
+	    baseline: Baseline stack outputs containing expected ALB DNS name
+
+	Returns:
+	    True if the verification endpoint returns the expected token
+	"""
+
+	# Compute expected token (matches what CDK generates)
+	expected_token = hashlib.sha256(
+		f"{baseline.deployment_name}:{baseline.alb_dns_name}".encode()
+	).hexdigest()[:16]
+	verification_path = f"/_pulse/verify-{expected_token}"
+
+	try:
+		async with httpx.AsyncClient(timeout=10.0, verify=True) as client:
+			response = await client.get(
+				f"https://{domain}{verification_path}",
+				follow_redirects=True,
+			)
+			if response.status_code == 200:
+				data = response.json()
+				returned_token = data.get("token", "")
+				if returned_token == expected_token:
+					return True
+	except Exception:
+		pass
+	return False
+
+
 async def _ensure_certificate_ready(
 	domain: str,
 	provided_arn: str | None,
@@ -209,9 +239,22 @@ async def _ensure_domain_routing(
 		proxied = domain_uses_cloudflare_proxy(domain)
 		if proxied:
 			reporter.info(
-				f"{domain} appears to be served through Cloudflare; skipping direct ALB IP verification."
+				f"{domain} appears to be served through Cloudflare proxy. "
+				+ "Attempting to verify ALB reachability via HTTPS endpoint..."
 			)
-			return True, True
+			# Use HTTPS verification endpoint to confirm domain reaches our ALB
+			alb_reachable = await _verify_alb_via_https(domain, baseline)
+			if alb_reachable:
+				reporter.success(
+					f"{domain} successfully verified to reach ALB ({baseline.alb_dns_name}) via HTTPS."
+				)
+				return True, True
+			else:
+				reporter.warning(
+					f"{domain} could not reach ALB verification endpoint. "
+					+ "Ensure the domain is properly configured to route to the ALB."
+				)
+				return False, True
 		reporter.success(f"{domain} resolves to {baseline.alb_dns_name}; DNS is ready.")
 		return True, False
 
@@ -244,9 +287,22 @@ async def _ensure_domain_routing(
 			proxied = domain_uses_cloudflare_proxy(domain)
 			if proxied:
 				reporter.info(
-					f"{domain} appears to be served through Cloudflare; skipping direct ALB IP verification."
+					f"{domain} appears to be served through Cloudflare proxy. "
+					+ "Attempting to verify ALB reachability via HTTPS endpoint..."
 				)
-				return True, True
+				# Use HTTPS verification endpoint to confirm domain reaches our ALB
+				alb_reachable = await _verify_alb_via_https(domain, baseline)
+				if alb_reachable:
+					reporter.success(
+						f"{domain} successfully verified to reach ALB ({baseline.alb_dns_name}) via HTTPS."
+					)
+					return True, True
+				else:
+					reporter.warning(
+						f"{domain} could not reach ALB verification endpoint. "
+						+ "Ensure the domain is properly configured to route to the ALB."
+					)
+					return False, True
 			reporter.success(f"{domain} now resolves to {baseline.alb_dns_name}.")
 			return True, False
 
@@ -260,6 +316,7 @@ async def _ensure_domain_routing(
 
 async def build_and_push_image(
 	dockerfile_path: Path,
+	deployment_name: str,
 	deployment_id: str,
 	baseline: BaselineStackOutputs,
 	*,
@@ -271,10 +328,12 @@ async def build_and_push_image(
 
 	Args:
 	    dockerfile_path: Path to the Dockerfile
+	    deployment_name: Deployment environment name (e.g., "prod", "dev")
 	    deployment_id: Unique deployment ID to use as the image tag
 	    baseline: Baseline stack outputs containing ECR repository URI
 	    context_path: Path to the Docker build context directory
 	    build_args: Additional build arguments to pass to docker build
+	        (DEPLOYMENT_NAME and DEPLOYMENT_ID are automatically added)
 
 	Returns:
 	    The full image URI with tag (e.g., "123.dkr.ecr.us-east-1.amazonaws.com/repo:tag")
@@ -302,6 +361,7 @@ async def build_and_push_image(
 	# Build the image
 	reporter.info(f"Building image: {image_uri}")
 	all_build_args = {
+		"DEPLOYMENT_NAME": deployment_name,
 		"DEPLOYMENT_ID": deployment_id,
 		**(build_args or {}),
 	}
@@ -408,7 +468,6 @@ async def register_task_definition(
 	env_vars: dict[str, str] | None = None,
 	drain_poll_seconds: int = 5,
 	drain_grace_seconds: int = 20,
-	drain_health_fail_after_seconds: int = 120,
 	reporter: Reporter | None = None,
 ) -> str:
 	"""Register an ECS Fargate task definition.
@@ -421,8 +480,7 @@ async def register_task_definition(
 	    memory: Memory in MB (512, 1024, 2048, etc.)
 	    env_vars: Additional environment variables
 	    drain_poll_seconds: Seconds between SSM state polling
-	    drain_grace_seconds: Grace period before ShutdownReady=1
-	    drain_health_fail_after_seconds: Seconds before health fails when draining
+	    drain_grace_seconds: Grace period before marking task as draining
 
 	Returns:
 	    The ARN of the registered task definition
@@ -443,10 +501,6 @@ async def register_task_definition(
 		{"name": "PULSE_AWS_DEPLOYMENT_ID", "value": deployment_id},
 		{"name": "PULSE_AWS_DRAIN_POLL_SECONDS", "value": str(drain_poll_seconds)},
 		{"name": "PULSE_AWS_DRAIN_GRACE_SECONDS", "value": str(drain_grace_seconds)},
-		{
-			"name": "PULSE_AWS_DRAIN_HEALTH_FAIL_AFTER_SECONDS",
-			"value": str(drain_health_fail_after_seconds),
-		},
 		*[{"name": k, "value": v} for k, v in (env_vars or {}).items()],
 	]
 
@@ -510,7 +564,7 @@ async def set_deployment_state(
 	Args:
 	    deployment_name: The deployment environment name (e.g., "prod", "dev")
 	    deployment_id: Unique deployment ID
-	    state: Deployment state ("active" or "draining")
+	    state: Deployment state ("deploying", "active", or "draining")
 	    region: AWS region
 	    reporter: Optional reporter for logging
 
@@ -535,6 +589,47 @@ async def set_deployment_state(
 		raise DeploymentError(msg) from exc
 
 
+async def update_service_state_tag(
+	service_arn: str,
+	deployment_name: str,
+	deployment_id: str,
+	state: str,
+	*,
+	region: str,
+	reporter: Reporter | None = None,
+) -> None:
+	"""Update the state tag on an ECS service.
+
+	Args:
+	    service_arn: ARN of the ECS service
+	    deployment_name: The deployment environment name
+	    deployment_id: Unique deployment ID
+	    state: Deployment state ("deploying", "active", or "draining")
+	    region: AWS region
+	    reporter: Optional reporter for logging
+
+	Raises:
+	    DeploymentError: If tag update fails
+	"""
+	reporter = _resolve_reporter(reporter)
+
+	ecs = boto3.client("ecs", region_name=region)
+
+	try:
+		ecs.tag_resource(
+			resourceArn=service_arn,
+			tags=[
+				{"key": "deployment_name", "value": deployment_name},
+				{"key": "deployment_id", "value": deployment_id},
+				{"key": "state", "value": state},
+			],
+		)
+		reporter.detail(f"Updated service tag state = {state}")
+	except ClientError as exc:
+		msg = f"Failed to update service state tag: {exc}"
+		raise DeploymentError(msg) from exc
+
+
 async def mark_previous_deployments_as_draining(
 	deployment_name: str,
 	current_deployment_id: str,
@@ -598,10 +693,10 @@ async def mark_previous_deployments_as_draining(
 		raise DeploymentError(msg) from exc
 
 	# Mark each previous deployment as draining
-	drained_ids = []
+	drained_ids: list[str] = []
 
 	for svc in previous_services:
-		old_deployment_id = svc["serviceName"]
+		old_deployment_id = cast(str, svc["serviceName"])
 		service_arn = svc["serviceArn"]
 
 		reporter.detail(f"Marking {old_deployment_id} as draining...")
@@ -809,7 +904,7 @@ async def create_service_and_target_group(
 			tags=[
 				{"key": "deployment_id", "value": deployment_id},
 				{"key": "deployment_name", "value": deployment_name},
-				{"key": "state", "value": "active"},
+				{"key": "state", "value": "deploying"},
 			],
 		)
 		service_arn = service_response["service"]["serviceArn"]
@@ -927,7 +1022,10 @@ async def wait_for_healthy_targets(
 
 		try:
 			response = elbv2.describe_target_health(TargetGroupArn=target_group_arn)
-			targets = response.get("TargetHealthDescriptions", [])
+
+			targets = cast(
+				list[dict[str, Any]], response.get("TargetHealthDescriptions", [])
+			)
 			current_time = asyncio.get_event_loop().time()
 
 			healthy_count = sum(
@@ -943,7 +1041,7 @@ async def wait_for_healthy_targets(
 			total_count = len(targets)
 
 			# Track when tasks exit initial state and check grace periods
-			tasks_failed_grace_period = []
+			tasks_failed_grace_period: list[dict[str, Any]] = []
 			for t in targets:
 				target = t.get("Target", {})
 				target_id = target.get("Id", "")
@@ -1286,6 +1384,7 @@ async def cleanup_inactive_deployments(
 	# Create AWS clients
 	ecs = boto3.client("ecs", region_name=baseline.region)
 	elbv2 = boto3.client("elbv2", region_name=baseline.region)
+	ssm = boto3.client("ssm", region_name=baseline.region)
 
 	# Call the reaper's cleanup function
 	try:
@@ -1294,6 +1393,7 @@ async def cleanup_inactive_deployments(
 			listener_arn=baseline.listener_arn,
 			ecs=ecs,
 			elbv2=elbv2,
+			ssm_client=ssm,
 		)
 
 		if cleaned_count > 0:
@@ -1319,8 +1419,6 @@ async def deploy(
 	health_check: HealthCheckConfig | None = None,
 	reaper: ReaperConfig | None = None,
 	certificate_arn: str | None = None,
-	context: DeploymentContext | None = None,
-	reporter: Reporter | None = None,
 ) -> dict[str, str]:
 	"""Deploy an application to AWS ECS with full blue-green deployment workflow.
 
@@ -1347,8 +1445,6 @@ async def deploy(
 	    health_check: ALB health check configuration (uses defaults if None)
 	    reaper: Reaper Lambda configuration (uses defaults if None)
 	    certificate_arn: ACM certificate ARN (looked up if not provided)
-	    context: Optional deployment execution context (auto-detected if omitted)
-	    reporter: Optional reporter to override logging behaviour
 
 	Returns:
 	    Dictionary with deployment information:
@@ -1392,7 +1488,7 @@ async def deploy(
 	"""
 	from pulse_aws.baseline import ensure_baseline_stack
 
-	context = _prepare_context(context, reporter)
+	context = create_context()
 	reporter = context.reporter
 
 	reporter.section(f"Deploy {deployment_name}")
@@ -1423,6 +1519,7 @@ async def deploy(
 	# Build and push image
 	image_uri = await build_and_push_image(
 		dockerfile_path=docker.dockerfile_path,
+		deployment_name=deployment_name,
 		deployment_id=deployment_id,
 		baseline=baseline,
 		context_path=docker.context_path,
@@ -1441,7 +1538,6 @@ async def deploy(
 		env_vars=task.env_vars or None,
 		drain_poll_seconds=task.drain_poll_seconds,
 		drain_grace_seconds=task.drain_grace_seconds,
-		drain_health_fail_after_seconds=task.drain_health_fail_after_seconds,
 		reporter=reporter,
 	)
 
@@ -1460,16 +1556,16 @@ async def deploy(
 		reporter=reporter,
 	)
 
-	# Mark this deployment as active in SSM
+	# Mark this deployment as deploying in SSM (to prevent reaper from cleaning it up)
 	await set_deployment_state(
 		deployment_name=deployment_name,
 		deployment_id=deployment_id,
-		state="active",
+		state="deploying",
 		region=baseline.region,
 		reporter=reporter,
 	)
 
-	# Install listener rules and switch traffic
+	# Install listener rules and switch traffic (waits for health confirmation)
 	await install_listener_rules_and_switch_traffic(
 		deployment_name=deployment_name,
 		deployment_id=deployment_id,
@@ -1482,6 +1578,23 @@ async def deploy(
 		reporter=reporter,
 	)
 
+	# Mark this deployment as active in SSM and update service tag (only after health confirmed and traffic switched)
+	await set_deployment_state(
+		deployment_name=deployment_name,
+		deployment_id=deployment_id,
+		state="active",
+		region=baseline.region,
+		reporter=reporter,
+	)
+	await update_service_state_tag(
+		service_arn=service_arn,
+		deployment_name=deployment_name,
+		deployment_id=deployment_id,
+		state="active",
+		region=baseline.region,
+		reporter=reporter,
+	)
+
 	# Mark previous deployments as draining in SSM and service tags
 	reporter.section("Post-Deployment")
 	marked_draining = await mark_previous_deployments_as_draining(
@@ -1512,7 +1625,8 @@ async def deploy(
 	reporter.detail(f"Image URI: {image_uri}")
 	if domain_proxied:
 		reporter.detail(
-			f"{domain} is served via Cloudflare proxy; ALB IP verification skipped."
+			f"{domain} is served via Cloudflare proxy. "
+			+ "ALB reachability verified via HTTPS endpoint."
 		)
 	if marked_draining:
 		reporter.detail(
@@ -1546,5 +1660,6 @@ __all__ = [
 	"mark_previous_deployments_as_draining",
 	"register_task_definition",
 	"set_deployment_state",
+	"update_service_state_tag",
 	"wait_for_healthy_targets",
 ]
